from django.shortcuts import render
from django.urls import path, reverse
from django.views.decorators.csrf import csrf_exempt
from graphene_django.views import GraphQLView

from .settings import grapple_settings, has_channels


def graphiql(request):
    graphiql_settings = {
        "REACT_VERSION": "16.14.0",
        "GRAPHIQL_VERSION": "1.11.5",
        "endpointURL": reverse("grapple_graphql"),
        "supports_subscriptions": has_channels,
    }
    if has_channels:
        # TODO: The version below is fixed at 0.8.3 and could be upgraded, but
        # this need to be done with caution as 0.9.x breaks GraphiQL 1.4.2.
        graphiql_settings["SUBSCRIPTIONS_TRANSPORT_VERSION"] = "0.8.3"
        graphiql_settings["subscriptionsEndpoint"] = "ws://localhost:8000/subscriptions"

    return render(request, "grapple/graphiql.html", graphiql_settings)


# Traditional URL routing
urlpatterns = [
    path("graphql/", csrf_exempt(GraphQLView.as_view()), name="grapple_graphql")
]

if grapple_settings.EXPOSE_GRAPHIQL:
    urlpatterns.append(path("graphiql/", graphiql, name="grapple_graphiql"))



import graphene
from graphql.error import GraphQLError

try:
    from wagtail.models import Site
except ImportError:
    from wagtail.core.models import Site

from ..registry import registry
from ..utils import resolve_site

try:
    from wagtail.contrib.settings.models import BaseSiteSetting
except ImportError:
    # Wagtail < 4.0
    from wagtail.contrib.settings.models import BaseSetting as BaseSiteSetting


def SettingsQuery():
    if registry.settings:

        class SettingsObjectType(graphene.Union):
            class Meta:
                types = registry.settings.types

        class Mixin:
            setting = graphene.Field(
                SettingsObjectType,
                name=graphene.String(),
                site=graphene.String(),
            )
            settings = graphene.List(
                graphene.NonNull(SettingsObjectType),
                required=True,
                name=graphene.String(),
                site=graphene.String(),
            )

            # Return just one setting base on name param.
            def resolve_setting(self, info, **kwargs):
                # Site filter
                # Only applies to settings that inherit from BaseSiteSetting
                site_hostname = kwargs.pop("site", None)

                if site_hostname is not None:
                    try:
                        site = resolve_site(site_hostname)
                    except Site.MultipleObjectsReturned:
                        raise GraphQLError(
                            "Your 'site' filter value of '{}' returned multiple sites. Try adding a port number (for example: '{}:80').".format(
                                site_hostname, site_hostname
                            )
                        )
                else:
                    site = None

                name = kwargs.get("name")
                for setting in registry.settings:
                    # If 'name' filter used, ignore any models that don't match the filter
                    if name and setting._meta.model_name != name.lower():
                        continue

                    if site and issubclass(setting._meta.model, BaseSiteSetting):
                        return setting._meta.model.objects.filter(site=site).first()
                    else:
                        return setting._meta.model.objects.first()

            # Return all settings.
            def resolve_settings(self, info, **kwargs):
                # Site filter
                # Only applies to settings that inherit from BaseSiteSetting
                site_hostname = kwargs.pop("site", None)

                if site_hostname is not None:
                    try:
                        site = resolve_site(site_hostname)
                    except Site.MultipleObjectsReturned:
                        raise GraphQLError(
                            "Your 'site' filter value of '{}' returned multiple sites. Try adding a port number (for example: '{}:80').".format(
                                site_hostname, site_hostname
                            )
                        )
                else:
                    site = None

                name = kwargs.get("name")
                settings_objects = []
                for setting in registry.settings:
                    # If 'name' filter used, ignore any models that don't match the filter
                    if name and setting._meta.model_name != name.lower():
                        continue

                    if site and issubclass(setting._meta.model, BaseSiteSetting):
                        settings_objects.extend(
                            setting._meta.model.objects.filter(site=site)
                        )

                    else:
                        settings_objects.extend(setting._meta.model.objects.all())

                return settings_objects

        return Mixin

    else:

        class Mixin:
            pass

        return Mixin



# Generated by Django 2.2.3 on 2019-08-22 09:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("grapple", "0001_initial")]

    operations = [migrations.DeleteModel(name="PagePreview")]



import graphene
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import gettext_lazy as _
from graphene_django.types import DjangoObjectType
from graphql.error import GraphQLError

try:
    from wagtail.models import Page as WagtailPage
    from wagtail.models import Site
except ImportError:
    from wagtail.core.models import Page as WagtailPage
    from wagtail.core.models import Site

from ..utils import resolve_queryset, resolve_site
from .pages import PageInterface, get_specific_page
from .structures import QuerySetList


class SiteObjectType(DjangoObjectType):
    pages = QuerySetList(
        graphene.NonNull(lambda: PageInterface),
        content_type=graphene.Argument(
            graphene.String,
            description=_("Filter by content type. Uses the `app.Model` notation."),
        ),
        enable_search=True,
        required=True,
    )
    page = graphene.Field(
        PageInterface,
        id=graphene.Int(),
        slug=graphene.String(),
        url_path=graphene.String(),
        token=graphene.String(),
        content_type=graphene.String(),
    )

    def resolve_pages(self, info, **kwargs):
        pages = WagtailPage.objects.in_site(self).live().public().specific()

        content_type = kwargs.pop("content_type", None)
        if content_type:
            app_label, model = content_type.strip().lower().split(".")
            try:
                ctype = ContentType.objects.get(app_label=app_label, model=model)
            except ContentType.DoesNotExist:
                return (
                    WagtailPage.objects.none()
                )  # something not quite right here, bail out early
            else:
                pages = pages.filter(content_type=ctype)

        return resolve_queryset(pages, info, **kwargs)

    def resolve_page(self, info, **kwargs):
        return get_specific_page(
            id=kwargs.get("id"),
            slug=kwargs.get("slug"),
            url_path=kwargs.get("url_path"),
            token=kwargs.get("token"),
            content_type=kwargs.get("content_type"),
            site=self,
        )

    class Meta:
        model = Site


def SitesQuery():
    class Mixin:
        site = graphene.Field(
            SiteObjectType, hostname=graphene.String(), id=graphene.ID()
        )
        sites = QuerySetList(
            graphene.NonNull(SiteObjectType), enable_search=True, required=True
        )

        # Return all sites.
        def resolve_sites(self, info, **kwargs):
            return resolve_queryset(Site.objects.all(), info, **kwargs)

        # Return a site, identified by ID or hostname.
        def resolve_site(self, info, **kwargs):
            id, hostname = kwargs.get("id"), kwargs.get("hostname")

            if id:
                return Site.objects.filter(pk=id).first()
            elif hostname:
                try:
                    return resolve_site(hostname)
                except Site.MultipleObjectsReturned:
                    raise GraphQLError(
                        "Your 'hostname' filter value of '{}' returned multiple sites. Try adding a port number (for example: '{}:80').".format(
                            hostname, hostname
                        )
                    )

    return Mixin



from django.apps import apps
from graphene import ObjectType

try:
    from wagtail import hooks
except ImportError:
    from wagtail.core import hooks

from .registry import registry
from .types.collections import CollectionsQuery
from .types.documents import DocumentsQuery
from .types.images import ImagesQuery
from .types.pages import PagesQuery
from .types.redirects import RedirectsQuery
from .types.search import SearchQuery
from .types.settings import SettingsQuery
from .types.sites import SitesQuery
from .types.snippets import SnippetsQuery
from .types.tags import TagsQuery


@hooks.register("register_schema_query")
def register_schema_query(query_mixins):
    query_mixins += [
        ObjectType,
        PagesQuery(),
        SitesQuery(),
        ImagesQuery(),
        DocumentsQuery(),
        SnippetsQuery(),
        SettingsQuery(),
        SearchQuery(),
        TagsQuery(),
        CollectionsQuery(),
        RedirectsQuery,
    ]

    if apps.is_installed("wagtailmedia"):
        from .types.media import MediaQuery

        query_mixins.append(MediaQuery())

    query_mixins += registry.schema


@hooks.register("register_schema_mutation")
def register_schema_mutation(mutation_mixins):
    mutation_mixins += [ObjectType]


@hooks.register("register_schema_subscription")
def register_schema_subscription(subscription_mixins):
    from .settings import has_channels

    if has_channels:
        from .types.pages import PagesSubscription

        subscription_mixins += [ObjectType, PagesSubscription()]



import inspect

from graphene import ResolveInfo
from graphql.execution.middleware import MIDDLEWARE_RESOLVER_FUNCTION

from .registry import registry

ROOT_TYPES = ["Query", "Mutation", "Subscription"]


def get_middleware_resolvers(middlewares):
    for middleware in middlewares:
        if inspect.isfunction(middleware):
            yield middleware
        if not hasattr(middleware, MIDDLEWARE_RESOLVER_FUNCTION):
            raise Exception(
                "Middleware must be either a class or a function. Got: {}.\nYou can read more about middleware here: https://docs.graphene-python.org/en/latest/execution/middleware/".format(
                    type(middleware)
                )
            )
        yield getattr(middleware(), MIDDLEWARE_RESOLVER_FUNCTION)


class IsAuthenticatedMiddleware(object):
    def resolve(self, next, root, info, **args):
        if not info.context.user.is_authenticated:
            return None
        return next(root, info, **args)


class IsAnonymousMiddleware(object):
    def resolve(self, next, root, info, **args):
        if not info.context.user.is_anonymous:
            return None
        return next(root, info, **args)


class GrappleMiddleware(object):
    def __init__(self):
        self.field_middlewares = {}
        for field_name in registry.field_middlewares:
            self.field_middlewares[field_name] = list(
                get_middleware_resolvers(registry.field_middlewares[field_name])
            )

    def resolve(self, next, root, info: ResolveInfo, **args):
        field_name = info.field_name
        parent_name = info.parent_type.name
        if field_name in self.field_middlewares and parent_name in ROOT_TYPES:
            for middleware in self.field_middlewares[field_name]:
                return middleware(next, root, info, **args)

        return next(root, info, **args)
